/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 *
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * ShapeByNameAction.cpp
 *
 *  Created on: Sep 14, 2012
 *      Author: ankele
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Shapes/ShapeRegistry.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"

#include <iostream>
#include <string>

#include "ShapeByNameAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "ShapeByNameAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr SelectionShapeByNameAction::performCall() {
  const Shape *Walker = ShapeRegistry::getInstance().getByName(params.shapeName.get());
  if (Walker != NULL) {
    if (!ShapeRegistry::getInstance().isSelected(params.shapeName.get())) {
      LOG(1, "Selecting shape " << params.shapeName.get());
      ShapeRegistry::getInstance().selectShape(params.shapeName.get());
      LOG(0, ShapeRegistry::getInstance().countSelectedShapes() << " shapes selected.");
      return Action::state_ptr(new SelectionShapeByNameState(params));
    } else {
      return Action::success;
    }
  } else {
    return Action::failure;
  }
}

Action::state_ptr SelectionShapeByNameAction::performUndo(Action::state_ptr _state) {
  SelectionShapeByNameState *state = assert_cast<SelectionShapeByNameState*>(_state.get());

  ShapeRegistry::getInstance().unselectShape(state->params.shapeName.get());

  return Action::state_ptr(_state);
}

Action::state_ptr SelectionShapeByNameAction::performRedo(Action::state_ptr _state){
  SelectionShapeByNameState *state = assert_cast<SelectionShapeByNameState*>(_state.get());

  ShapeRegistry::getInstance().selectShape(state->params.shapeName.get());

  return Action::state_ptr(_state);
}

bool SelectionShapeByNameAction::canUndo() {
  return true;
}

bool SelectionShapeByNameAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
